@@ -8,7 +8,6 @@ from collections import Counter
 import getopt
 import math
 import matplotlib.pyplot as plt
-from matplotlib.backends.backend_pdf import PdfPages
 import numpy as np
 import os
 import pandas as pd
@@ -188,7 +187,6 @@ def get_cpt_codes(path, filename, reg_type):
 	return phenotypes
 
 
-
 def generate_feature_matrix(genotypes, phenotype, reg_type, code_type, pheno_cov=None):
 	"""
 	Generates the feature matrix that will be used to run the PheWAS or ProWAS analysis. Feature matrix is 3xNxP,
@@ -319,21 +317,23 @@ def get_phenotype_info(p_index, code_type):
 	if code_type == 'ICD':
 		p_code = phewas_codes.loc[p_index, 'PheCode']
 		p_name = phewas_codes.loc[p_index, 'Phenotype']
+		p_cat = phewas_codes.loc[p_index, 'category_string']
 		icd9_ix = icd9_codes['PheCode'] == p_code
 		icd10_ix = icd10_codes['PheCode'] == p_code
-	
+
 		p_icd9_rollup = '/'.join(icd9_codes.loc[icd9_ix, 'ICD_CODE'].tolist())
 		p_icd10_rollup = '/'.join(icd10_codes.loc[icd10_ix, 'ICD_CODE'].tolist())
-		
-		return [p_code, p_name, p_icd9_rollup, p_icd10_rollup]
-	
+
+		return [p_code, p_name, p_cat, p_icd9_rollup, p_icd10_rollup]
+
 	else:
 		p_code = prowas_codes.loc[p_index, 'prowas_code']
 		p_name = prowas_codes.loc[p_index, 'prowas_desc']
+		p_cat = prowas_codes.loc[p_index, 'CCS Label']
 		cpt_ix = cpt_codes['prowas_code'] == p_code
 		p_cpt_rollup = '/'.join(cpt_codes.loc[cpt_ix, 'CPT_CODE'].tolist())
-		
-		return [p_code, p_name, p_cpt_rollup]
+
+		return [p_code, p_name, p_cat, p_cpt_rollup]
 
 
 def fit_pheno_model(genotypes, phen_vector1, phen_vector2, phen_vector3='', covariates='',
@@ -381,7 +381,7 @@ def fit_pheno_model(genotypes, phen_vector1, phen_vector2, phen_vector3='', cova
 	data['y'] = phen_vector1 # aggregate phenotype data
 
 	# append '+' to covariates (if there are any) -> makes definition of 'f' more elegant
-	if covariates is not '':
+	if covariates != '':
 		covariates = '+' + covariates
 
 	# add MaxAgeAtEvent to data (if needed)
@@ -423,23 +423,23 @@ def fit_pheno_model(genotypes, phen_vector1, phen_vector2, phen_vector3='', cova
 
 	except ValueError as ve:
 		print('\n')
-		if phenotype is not '':
+		if phenotype != '':
 			print('ERROR computing regression for phenotype %s (%s)' %(phenotype[0],phenotype[1]))
 		print(ve)
 		print('lr = % d' %lr)
 		reg_result = [np.nan, np.nan, np.nan, np.nan, np.nan]
 	except Exception as e:
 		print('\n')
-		if phenotype is not '':
+		if phenotype != '':
 			print('ERROR computing regression for phenotype %s (%s)' %(phenotype[0],phenotype[1]))
 		print(e)
 		reg_result = [np.nan, np.nan, np.nan, np.nan, np.nan]
 	return reg_result
 
 
-def run_phewas(fm, genotypes, code_type, covariates='', response='genotype'):
+def run_phewas_legacy(fm, genotypes, code_type, covariates='', response='genotype', phe_thresh=5):
 	"""
-	Run mass phenotype regressions
+	Run mass phenotype regressions (legacy version)
 
 	Iterate over all PheWAS/ProWAS codes in the feature matrix, running a logistic regression of the form:
 
@@ -460,18 +460,20 @@ def run_phewas(fm, genotypes, code_type, covariates='', response='genotype'):
 	:param code_type:  type of EMR code ('ICD' or 'CPT')
 	:param covariates: *[optional]* covariates to include in the regressions separated by '+' (e.g. 'sex+ageAtDx')
 	:param response: *[optional]* response variable in the logisitc model (default: *genotype*)
+	:param phe_thresh: *[optional]* threshold for running regression; see note (default: *5*)
 	:type fm: numpy array
 	:type genotypes: pandas DataFrame
 	:type code_type: str
 	:type covariates: str
 	:type response: str
+	:type phe_thresh: int
 
 	:returns: regression results for each phenotype
 	:rtype: pandas DataFrame
 
 
 	.. note:: To prevent false positives & improve statistical power, regressions are only computed for
-		  phenotypes which present for greater than 5 subjects in the cohort. Phenotypes which do not meet
+		  phenotypes which present for greater than ``phe_thresh`` subjects in the cohort. Phenotypes which do not meet
 		  this criteria are not included in the returned regression results.
 
 	.. note:: For phenotypes that present in both the case (``response`` = 1) and control (``response`` = 0) groups, maximum
@@ -508,8 +510,8 @@ def run_phewas(fm, genotypes, code_type, covariates='', response='genotype'):
 		phen_vector1 = fm[0][:, index]
 		phen_vector2 = fm[1][:, index]
 		phen_vector3 = fm[2][:, index]
-		# to prevent false positives, only run regressions if more than 5 records have positive values
-		if np.where(phen_vector1 > 0)[0].shape[0] > 5:
+		# to prevent false positives, only run regressions if more than phe_thresh records have positive values
+		if np.where(phen_vector1 > 0)[0].shape[0] > phe_thresh:
 			if index in inds: # decide whether or not to use regularization in logistic regression
 				use_regular = 1
 			else:
@@ -522,12 +524,94 @@ def run_phewas(fm, genotypes, code_type, covariates='', response='genotype'):
 
 		# save regression data
 		info = phen_info[0:2] + stat_info + phen_info[2:]
-			
+
 		regressions.loc[index] = info
 
 	return regressions.dropna(subset=['p-val']).sort_values(by='p-val') # sort by significance
 
 
+
+def run_phewas(fm, genotypes, code_type, covariates='', response='genotype', phe_thresh=5):
+	"""
+	Run mass phenotype regressions
+
+	Iterate over all PheWAS/ProWAS codes in the feature matrix, running a logistic regression of the form:
+
+	:math:`Pr(response) \sim logit(phenotype\_aggregate + covariates)`
+
+	``fm`` is a 3xNxP matrix, where N = number of subjects and P = number of PheWAS/ProWAS Codes; this should only
+	be consutrcted by ``pyPheWAS.pyPhewasCorev2.generate_feature_matrix`` - otherwise results will be untrustworthy.
+	To use the age feature matrix (``fm[1]``), include 'MaxAgeAtICD' or 'MaxAgeAtCPT' in the ``covariates`` string.
+	Other than 'MaxAgeAtICD' and 'MaxAgeAtCPT', all covariates and the response variable must be included in
+	the group DataFrame.
+
+	The returned DataFrame includes the PheWAS/ProWAS code, Phenotype (code description, e.g. 'Pain in joint'),
+	-log\ :sub:`10`\ (p-value), p-value, beta, beta's confidence interval, beta's standard error, and lists of the ICD-9/ICD-10 or
+	CPT codes that map to the phenotype.
+
+	:param fm: phenotype feature matrix derived via ``pyPheWAS.pyPhewasCorev2.generate_feature_matrix``
+	:param genotypes: group data
+	:param code_type:  type of EMR code ('ICD' or 'CPT')
+	:param covariates: *[optional]* covariates to include in the regressions separated by '+' (e.g. 'sex+ageAtDx')
+	:param response: *[optional]* response variable in the logisitc model (default: *genotype*)
+	:param phe_thresh: *[optional]* threshold for running regression; see note (default: *5*)
+	:type fm: numpy array
+	:type genotypes: pandas DataFrame
+	:type code_type: str
+	:type covariates: str
+	:type response: str
+	:type phe_thresh: int
+
+	:returns: regression results for each phenotype
+	:rtype: pandas DataFrame
+
+
+	.. note:: To prevent false positives & improve statistical power, regressions are only computed for
+		  phenotypes which present for greater than ``phe_thresh`` subjects in the cohort. Phenotypes which do not meet
+		  this criteria are not included in the returned regression results.
+
+	"""
+
+	# sort group data by id
+	print('Sorting group data...')
+	genotypes.sort_values(by='id', inplace=True)
+	genotypes.reset_index(inplace=True, drop=True)
+
+	num_pheno = len(fm[0, 0])
+	# check number of phenotypes in the feature matrix
+	if code_type == 'ICD':
+		assert num_pheno == phewas_codes.shape[0], "Expected %d columns in PheWAS feature matrix, but found %d. Please check the feature matrix" % (phewas_codes.shape[0], num_pheno)
+	else:
+		assert num_pheno == prowas_codes.shape[0], "Expected %d columns in ProWAS feature matrix, but found %d. Please check the feature matrix" % (prowas_codes.shape[0], num_pheno)
+
+	# store all of the pertinent data from the regressions
+	if code_type == 'ICD':
+		out_cols = phewas_reg_cols
+	else:
+		out_cols = prowas_reg_cols
+	regressions = pd.DataFrame(columns=out_cols)
+
+	for index in tqdm(range(num_pheno), desc='Running Regressions'):
+		phen_info = get_phenotype_info(index, code_type)
+		phen_vector1 = fm[0][:, index]
+		phen_vector2 = fm[1][:, index]
+		phen_vector3 = fm[2][:, index]
+		# to prevent false positives, only run regressions if more than phe_thresh records have positive values
+		if np.where(phen_vector1 > 0)[0].shape[0] > phe_thresh:
+			stat_info = fit_pheno_model(genotypes, phen_vector1, phen_vector2, phen_vector3, covariates,
+										response, phenotype=phen_info[0:2], lr=1, code_type=code_type)
+		else:
+			# not enough samples to run regression
+			stat_info = [np.nan, np.nan, np.nan, np.nan, np.nan]
+
+		# save regression data
+		info = phen_info[0:2] + stat_info + phen_info[2:]
+
+		regressions.loc[index] = info
+
+	return regressions.dropna(subset=['p-val']).sort_values(by='p-val')  # sort by significance
+
+
 """
 
 Result Visualization
@@ -659,7 +743,7 @@ def get_x_label_positions(categories, lines=True):
 	return label_positions
 
 
-def plot_manhattan(regressions, thresh, code_type='ICD', show_imbalance=True, save='', save_format=''):
+def plot_manhattan(regressions, thresh, code_type='ICD', show_imbalance=True, plot_all_pts=True, save='', save_format=''):
 	"""
 	Plots significant phenotype data on a Manhattan Plot.
 
@@ -673,6 +757,7 @@ def plot_manhattan(regressions, thresh, code_type='ICD', show_imbalance=True, sa
 	:param thresh: p-value significance threshold
 	:param code_type: type of EMR data ('ICD' or 'CPT')
 	:param show_imbalance: boolean variable that determines whether or not to show imbalances on the plot (default True)
+	:param plot_all_pts: plot all points regardless of significance (default True)
 	:param save: the output file to save to (if empty, display the plot)
 	:param save_format: format of the save file
 	:type regressions: pandas DataFrame
@@ -689,21 +774,26 @@ def plot_manhattan(regressions, thresh, code_type='ICD', show_imbalance=True, sa
 	ax = plt.subplot(111)
 	frame1 = plt.gca()
 
-	# Merge regressions with phenotype data to get categories
 	if code_type == 'ICD':
 		pheno_codes = phewas_codes
 		reg_key = 'PheWAS Code'
 		map_key = 'PheCode'
 		cat_key = 'category'
-		pheno_name = 'Phenotype'
+		cat_label = 'category_string'
+		pheno_name = 'PheWAS Name'
 	else:
 		pheno_codes = prowas_codes
 		reg_key = 'ProWAS Code'
 		map_key = 'prowas_code'
 		cat_key = 'ccs'
-		pheno_name = 'prowas_desc'
-		
-	regressions = pd.merge(regressions, pheno_codes, left_on=reg_key, right_on=map_key).sort_values(by=cat_key)
+		cat_label = 'CCS Label'
+		pheno_name = 'ProWAS Name'
+
+	if not cat_key in regressions.columns:
+		# Merge regressions with phenotype data to get categories
+		regressions = pd.merge(regressions, pheno_codes, left_on=reg_key, right_on=map_key, suffixes=[None, '_y'])
+
+	regressions.sort_values(by=cat_key, inplace=True)
 
 	# Plot all points w/ labels
 	e = 1 # x-axis counter
@@ -714,20 +804,25 @@ def plot_manhattan(regressions, thresh, code_type='ICD', show_imbalance=True, sa
 
 	for ix,data in regressions.iterrows():
 		logp_ix = data['"-log(p)"']
-		if  data['p-val'] < thresh:
-			# determine marker type based on whether/not showing imbalance
-			if show_imbalance:
-				mew = 1.5
-				if data['beta'] > 0: m = '+'
-				else: m = '_'
-			else:
-				mew = 0.0
-				m = 'o'
-			# Plot PheCode data point & format PheCode label
-			if code_type == 'ICD': c = cat_colors[data['category_string']]
-			else: c = 'xkcd:aqua' # constant color
+		# determine marker type based on whether/not showing imbalance
+		if show_imbalance:
+			mew = 1.5
+			if data['beta'] > 0: m = '+'
+			else: m = '_'
+		else:
+			mew = 0.0
+			m = 'o'
+		# Plot PheCode data point & format PheCode label
+		if code_type == 'ICD': c = cat_colors[data[cat_label]]
+		else: c = 'xkcd:aqua' # constant color
+		if data['p-val'] < thresh:
+			# plot significant PheCode/ProCode with label
+			ax.plot(e, logp_ix, m, color=c, fillstyle='full', markeredgewidth=mew)
+			artists.append(ax.text(e, logp_ix, data[pheno_name], rotation=45, va='bottom', fontsize=6))
+			e += 15
+		elif plot_all_pts:
+			# plot insignificant PheCode/ProCode without label
 			ax.plot(e, logp_ix, m, color=c, fillstyle='full', markeredgewidth=mew)
-			artists.append(ax.text(e, logp_ix, data[pheno_name], rotation=89, va='bottom', fontsize=6))
 			e += 15
 
 	# Category Legend
@@ -756,7 +851,7 @@ def plot_manhattan(regressions, thresh, code_type='ICD', show_imbalance=True, sa
 	return
 
 
-def plot_odds_ratio(regressions, thresh, code_type='ICD', save='', save_format='', label_loc="plot"):
+def plot_log_odds_ratio(regressions, thresh, code_type='ICD', save='', save_format='', label_loc="plot"):
 	"""
 	Plots significant phenotype data on a Log Odds Plot.
 
@@ -785,21 +880,26 @@ def plot_odds_ratio(regressions, thresh, code_type='ICD', save='', save_format='
 	ax = plt.subplot(111)
 	frame1 = plt.gca()
 
-	# Merge regressions with phenotype data to get categories
 	if code_type == 'ICD':
 		pheno_codes = phewas_codes
 		reg_key = 'PheWAS Code'
 		map_key = 'PheCode'
 		cat_key = 'category'
-		pheno_name = 'Phenotype'
+		cat_label = 'category_string'
+		pheno_name = 'PheWAS Name'
 	else:
 		pheno_codes = prowas_codes
 		reg_key = 'ProWAS Code'
 		map_key = 'prowas_code'
 		cat_key = 'ccs'
-		pheno_name = 'prowas_desc'
-	
-	regressions = pd.merge(regressions, pheno_codes, left_on=reg_key, right_on=map_key).sort_values(by=cat_key)
+		cat_label = 'CCS Label'
+		pheno_name = 'ProWAS Name'
+
+	if not cat_key in regressions.columns:
+		# Merge regressions with phenotype data to get categories
+		regressions = pd.merge(regressions, pheno_codes, left_on=reg_key, right_on=map_key, suffixes=[None, '_y'])
+
+	regressions.sort_values(by=cat_key, inplace=True)
 
 
 	# Plot all points w/ labels
@@ -824,7 +924,7 @@ def plot_odds_ratio(regressions, thresh, code_type='ICD', save='', save_format='
 				phecode_locs.append(e)
 
 			# Plot Phecode Data
-			if code_type == 'ICD': c = cat_colors[data['category_string']]
+			if code_type == 'ICD': c = cat_colors[data[cat_label]]
 			else: c = 'xkcd:aqua' # constant color
 			ax.plot(beta_ix, e, 'o', color=c, fillstyle='full', markeredgewidth=0.0)
 			ax.plot([data['lowlim'], data['uplim']], [e, e], color=c)
@@ -895,7 +995,7 @@ def plot_volcano(regressions, code_type='ICD', save='', save_format=''):
 	artists = []
 	plt.ylabel('-log10(p)')
 	plt.xlabel('Log Odds Ratio')
-	
+
 	if code_type == 'ICD':
 		pheno_name = 'PheWAS Name'
 	else:
@@ -977,6 +1077,7 @@ phewas_reg_cols = ['PheWAS Code',
 				  'beta',
 				  'Conf-interval beta',
 				  'std_error',
+				  'category_string',
 				  'ICD-9',
 				  'ICD-10']
 
@@ -987,6 +1088,7 @@ prowas_reg_cols = ['ProWAS Code',
 				  'beta',
 				  'Conf-interval beta',
 				  'std_error',
+				  'CCS Label',
 				  'CPT']
 
 cat_colors = {'other': 'gold',
